﻿using System;
using System.Threading;
using OMR.TableDataSync.Console.Configuration;
using OMR.TableDataSync.Console.Helpers;
using OMR.TableDataSync.Core;
using System.Linq;
using System.Collections.Generic;
using OMR.TableDataSync.Core.Helpers;

namespace OMR.TableDataSync.Console
{
    class Program
    {
        static void Main(string[] args)
        {
            ConsoleSpiner cs = new ConsoleSpiner();

            TraceHelper.OnWrite += new TraceHelper.OnWriteHandler(TraceHelper_OnWrite);
            TraceHelper.Write("Sync progress is initializing...\n");

            TableDataSyncConfiguration config = (TableDataSyncConfiguration)XmlHelper.ReadFromXml("SyncConfig.xml", typeof(TableDataSyncConfiguration));
            while (true)
            {
                _traceCount = 0;
                System.Console.ForegroundColor = System.ConsoleColor.White;
                System.Console.WriteLine("Syncronizing...");

                SyncManager.SyncAllDatabases(config);

                Thread.Sleep(10000);
            }

            System.Console.ReadKey();
        }

        static int _traceCount = 0;
        static void TraceHelper_OnWrite(TraceWriteEvent traceWriteEvent)
        {
            ConsoleColor color = (ConsoleColor)traceWriteEvent.Level;

            System.Console.ForegroundColor = color;
            System.Console.Write("{0}-{1: HH:mm:ss:ffff}:\t{2}", _traceCount++, DateTime.Now, traceWriteEvent.Message);
        }

        public class ConsoleSpiner
        {
            static int counter;
            Thread th = new Thread(Turn);

            public ConsoleSpiner()
            {
                counter = 0;
                th.Start();
            }

            private static void Turn()
            {
                while (true)
                {
                    Spin();
                    Thread.Sleep(100);
                }
            }

            private static void Spin()
            {
                counter++;

                switch (counter % 4)
                {
                    case 0: System.Console.Write("|"); break;
                    case 1: System.Console.Write("/"); break;
                    case 2: System.Console.Write("-"); break;
                    case 3: System.Console.Write("\\"); break;
                }

                if (counter > 4) counter = 0;

                try
                {
                    System.Console.SetCursorPosition(System.Console.CursorLeft - 1, System.Console.CursorTop);
                }
                catch (Exception)
                {
                }
            }
        }
    }
}