﻿// 2011, Omer Faruk ZORLU
// http://www.omerfarukzorlu.com
using System;
using System.Data;
using System.Linq;
using System.Text;
using Microsoft.ApplicationBlocks.Data;
using OMR.CommerceSite.DataAccess;
using OMR.DataSync.Core.Helpers;
using OMR.TableDataSync.Console.Configuration;
using OMR.TableDataSync.Core.Domain;

namespace OMR.TableDataSync.Core.DataAccess
{
    public class DiscoverBLL : BaseBLL
    {
        private const string ChecksumAlias = "OMR_SYNC_CHKSM";

        public DiscoverBLL(string connectionString)
            : base(connectionString)
        { }

        public TableHash GetTableHash(TableConfiguration tableConfiguration)
        {
            ValidationHelper.AreNotNull(tableConfiguration);

            var primaryKeys = tableConfiguration.ColumnConfigurations.Where(k => k.IsPrimaryKey == true).ToList();
            var hashColumns = tableConfiguration.ColumnConfigurations.Where(k => k.IgnoreOnHash == false);

            if (0 == primaryKeys.Count())
            {
                throw new InvalidProgramException(string.Format("{0} must have a primary key"));
            }

            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(
                                "SELECT {0}, CHECKSUM({1}) AS {2} FROM {3} ORDER BY {4} ASC",
                                string.Join(", ", primaryKeys),
                                string.Join(", ", hashColumns),
                                ChecksumAlias,
                                tableConfiguration.Name,
                                string.Join(" ASC, ", primaryKeys)
                        );

            var sdr = SqlHelper.ExecuteReader(base.DbConnection, CommandType.Text, sb.ToString());

            TableHash tableHash = new TableHash();
            tableHash.PrimaryKeyNames.AddRange(primaryKeys.Select(k => k.Name));

            RowHash rowHash;
            while (sdr.Read())
            {
                rowHash = new RowHash();

                for (int i = 0; i < primaryKeys.Count(); i++)
                {
                    rowHash.PrimaryKeyValues.Add(sdr[i]);
                }

                rowHash.Hash = Convert.ToInt32(sdr[ChecksumAlias]);

                tableHash.RowHashes.Add(rowHash);
            }

            sdr.Close();

            return tableHash;
        }

        public static void ComputeTargets(TableHash source, TableHash destination)
        {
            ValidationHelper.AreNotNull(source, destination);

            int a = 0, b = 0;
            while (true)
            {
                if (
                    a == source.RowHashes.Count &&
                    b == destination.RowHashes.Count
                    )
                {
                    break;
                }

                // todo: max a,b
                if (a == source.RowHashes.Count)
                {
                    destination.RowHashes[b].Target = TableHashTargets.WillBeDeleted;
                    ++b;
                    continue;
                }

                var sRow = source.RowHashes[a];

                if (b == destination.RowHashes.Count)
                {
                    sRow.Target = TableHashTargets.WillBeInserted;
                    ++a;
                    continue;
                }

                var dRow = destination.RowHashes[b];

                // Check for are pk's equal
                int compareResult = 0; // 0= equal, -1= source less than destination, 1 source grather then source

                for (int j = 0; j < sRow.PrimaryKeyValues.Count; j++)
                {
                    compareResult = sRow.PrimaryKeyValues[j].CompareTo(dRow.PrimaryKeyValues[j]);

                    if (0 != compareResult)
                        break;
                }

                if (0 == compareResult)
                {
                    sRow.Target = sRow.Hash == dRow.Hash ? TableHashTargets.Equal : TableHashTargets.WillBeUpdated;

                    ++a;
                    ++b;
                }
                else if (0 < compareResult)
                {
                    dRow.Target = TableHashTargets.WillBeDeleted;
                    ++b;
                }
                else if (0 > compareResult)
                {
                    sRow.Target = TableHashTargets.WillBeInserted;
                    ++a;
                }

            }
        }
    }
}